@@ -79,43 +79,62 @@ class SocialAuthView(views.APIView):
         serializer = self.user_seriliser(user)
         return Response(serializer.data, status=status.HTTP_201_CREATED)
 
+
 import json
+import hmac
+from base64 import b64encode
+from hashlib import sha1
+from requests import HTTPError
+from social.exceptions import AuthCanceled, AuthTokenError
 from social.backends.oauth import BaseOAuth1
 from social.backends.linkedin import BaseLinkedinAuth
 
 
-class LinkedinOAuth(BaseLinkedinAuth, BaseOAuth1):
+class LinkedinJSAPIOAuth(BaseLinkedinAuth, BaseOAuth1):
     """Linkedin OAuth authentication backend"""
     name = 'linkedin'
     SCOPE_SEPARATOR = '+'
     AUTHORIZATION_URL = 'https://www.linkedin.com/uas/oauth/authenticate'
     REQUEST_TOKEN_URL = 'https://api.linkedin.com/uas/oauth/requestToken'
     ACCESS_TOKEN_URL = 'https://api.linkedin.com/uas/oauth/accessToken'
+    ACCESS_TOKEN_METHOD = 'POST'
 
-    # def user_data(self, access_token, *args, **kwargs):
-    #     """Return user data provided"""
-    #     return self.get_json(
-    #         self.user_details_url(),
-    #         params={'format': 'json'},
-    #         auth=self.oauth_auth(access_token),
-    #         headers=self.user_data_headers()
-    #     )
+    def user_data(self, access_token, *args, **kwargs):
+        """Return user data provided"""
+        return self.get_json(
+            self.user_details_url(),
+            params={'format': 'json'},
+            auth=self.oauth_auth(access_token),
+            headers=self.user_data_headers()
+        )
 
-    # def unauthorized_token(self):
-    #     """Makes first request to oauth. Returns an unauthorized Token."""
-    #     scope = self.get_scope() or ''
-    #     if scope:
-    #         scope = '?scope=' + self.SCOPE_SEPARATOR.join(scope)
-    #     return self.request(self.REQUEST_TOKEN_URL + scope,
-    #                         params=self.request_token_extra_arguments(),
-    #                         auth=self.oauth_auth()).content
+    def validate_bearer_token(self, token):
+        """ Validate token provided by LinkedIN JSAPI. """
+        data = ''.join([token[k] for k in token['signature_order']])
+        secret = self.setting('SECRET')
+        signature = b64encode(hmac.new(secret, data, sha1).digest())
+        return signature == token['signature']
 
     def do_auth(self, access_token, *args, **kwargs):
-        """Finish the auth process once the access_token was retrieved"""
+        """ Exchange bearer token before finishing auth. """
         access_token = json.loads(access_token)
 
-        data = self.user_data(access_token)
-        if data is not None and 'access_token' not in data:
-            data['access_token'] = access_token
-        kwargs.update({'response': data, 'backend': self})
-        return self.strategy.authenticate(*args, **kwargs)
+        if not self.validate_bearer_token(access_token):
+            raise AuthTokenError(self)
+
+        try:
+            access_token = self.access_token(access_token)
+        except HTTPError as err:
+            if err.response.status_code == 400:
+                raise AuthCanceled(self)
+            else:
+                raise
+
+        return super(LinkedinJSAPIOAuth, self).do_auth(access_token)
+
+    def access_token(self, token):
+        """Return request for access token value"""
+        data = {'xoauth_oauth2_access_token': token['access_token']}
+        return self.get_querystring(self.ACCESS_TOKEN_URL, data=data,
+                                    auth=self.oauth_auth(token),
+                                    method=self.ACCESS_TOKEN_METHOD)
