@@ -4,10 +4,10 @@ from mock import Mock
 from django.http import Http404
 from django_nose import FastFixtureTestCase as TestCase
 
-from .permissions import DenyCreateOnPutPermission
+from .permissions import DenyCreateOnPutPermission, NotAuthenticatedPermission
 
 
-class PermissionTest(TestCase):
+class PermissionsTest(TestCase):
     def test_deny_create_on_put_permission(self):
         permission = DenyCreateOnPutPermission()
         view = Mock()
@@ -22,3 +22,15 @@ class PermissionTest(TestCase):
         request.method = 'PUT'
         view.get_object = Mock(side_effect=Http404)
         self.assertFalse(permission.has_permission(request, view))
+
+    def test_not_authenticated_permission(self):
+        permission = NotAuthenticatedPermission()
+
+        view = Mock()
+        request = Mock()
+
+        request.user.is_authenticated = Mock(return_value=True)
+        self.assertFalse(permission.has_permission(request, view))
+
+        request.user.is_authenticated = Mock(return_value=False)
+        self.assertTrue(permission.has_permission(request, view))
